/**
 * Semestální práce č. 4 z předmětu KIV/VSP
 * (c) 2006 Ivan Habernal
 * e-mail: habernal@students.zcu.cz
 */
package cz.habi.vsp.jsim;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;

/**
 * This class represents a complex system simulation. The system has some
 * paralell working modules.
 * 
 * @author habi
 * 
 */
public class SystemSimulation extends JSimSimulation {
	static Logger log = Logger.getLogger(SystemSimulation.class);

	private static final String NAME = "VSP system simulation";

	private static final int MODULES_COUNT = 4;

	private static final int MINIMUM_MODULES_WORKING = 2;

	private double lambda;

	private double mu;

	private int steps;
	
	private Statistics statistics = new Statistics();

	Set<Module> modules = new HashSet<Module>();

	/**
	 * Creates a new instance of SystemSimulation
	 * 
	 * @param steps
	 *            number of simulation steps
	 * @param lambda
	 *            lambda coefficient
	 * @param mu
	 *            mu coeficcient
	 * @throws JSimException
	 */
	public SystemSimulation(int steps, double lambda, double mu) throws JSimException {
		super(NAME);
		this.steps = steps;
		this.lambda = lambda;
		this.mu = mu;

		for (int i = 0; i < MODULES_COUNT; i++) {
			Module m = new Module("module" + i, this, this.lambda, this.mu);
			m.activateNow();
			modules.add(m);
		}
	}

	/**
	 * The simulation body.
	 * 
	 * @throws JSimException
	 *             if an exception occures
	 */
	public void run() throws JSimException {
		for (int i = 0; i < steps; i++) {
			this.step();
			
			repair();

			boolean working = isWorking();
			double time = getCurrentTime();
			log.info("Step: " + i + ", time: " + time + ", working: " + working);
			
			statistics.addTime(working, time);
		}
	}

	/**
	 * Returns true, if the whole system is working (depends on the appropriate
	 * constant)
	 * 
	 * @return state of system
	 */
	private boolean isWorking() {
		int workingModules = 0;
		for (Module m: modules) {
			if (m.isWorking()) {
				workingModules++;
			}
		}
		
		return workingModules >= MINIMUM_MODULES_WORKING;
	}
	
	public Statistics getStatistics() {
		return statistics;
	}
	
	private static final int REPAIRING_AVAILABILITY = 1;
	
	private List<Module> repairingQueue = new ArrayList<Module>();
	
	public void addToQueue(Module module) {
		log.info("Module " + module.getName() + " added to repairing queue");
		repairingQueue.add(module);
	}
	
	private void repair() throws JSimException {
		if (repairingQueue.isEmpty()) {
			return;
		}
		
		for (int i = 0; (i < repairingQueue.size()) && (i < REPAIRING_AVAILABILITY); i++) {
			Module m = repairingQueue.remove(i);
			m.activateNow();
			log.info("Module " + m.getName() + " activated for repairing");
		}
	}
}
